        page    66,80
        TITLE   HGC!.COM Sets Hercules video modes
;       Version 1.4 1996/10/25 by Roedy Green
;       works with MASM 5.0 and Optasm
;==============================================================
Comment |

 HGC! 1.4 ۲

HGC! 1.4 is a Hercules video mode utility replacement for HGC.
For monochrome Hercules Graphics (GB102) and Graphics Plus
(GB112) Video Cards or close clones only.  If you use it on non
Herc cards, it simply displays an error message.

HGC!.COM is a simpler, smaller, non-TSR replacement for the
Hercules HGC utility.  For this reason HGC! does not support the
PRINT and SAVE options that Hercules' HGC does.

Try:
HGC!         -- displays help screen.
HGC! DIAG    -- to inhibit graphics.
HGC! HALF    -- to permit partial graphics.
HGC! FULL    -- to permit full graphics.
HGC! ROMFONT -- turns off fancy 4K or 48K RAMFONT modes
                and also clears the screen.

All command options may be abbreviated to a single letter and
you may combine modes such as HGC! ROMFONT HALF.

Note, HGC! FULL does not actually put you in graphics mode;
it simply controls the lock that grants access to full graphics.

This program was based on INIT.ASM and ROMFONT.ASM that comes
with the Hercules Plus GB112 graphics card.



Please report bugs and problems to:

Roedy Green
Canadian Mind Products
#208 - 525 Ninth Street
New Westminster BC Canada
V5H 2N6
tel:(604) 777-1804
mailto:roedy@mindprod.com
http://mindprod.com


Version 1.0 91/03/11

Version 1.1 91/03/11
- words RAMFONT and ROMFONT reversed in several places.
- program worked, but documentation was confusing.

Version 1.2 93/06/08
- embed new address and phone number

Version 1.3 1996 October 25
- embed POB 707 Quathiaski Cove address

Version 1.4 1998 November 8
- embed Barker address


| ; end of comment
;
;               port address
IndexPort       equ             03b4h
DataPort        equ             03B5h
CntrlPort       equ             03b8h
StatusPort      equ             03bah

AllowPort       equ             03bfh   ; controls level of graphics Allowed

;               control codes

scrn_off        equ             0       ; turn off video
scrn_on         equ             8       ; turn on video
textMode        equ             20h     ; code for text mode

id_mask         equ     00110000b       ; used in detecting HercPlus
id_code         equ     00010000b

;==============================================================

REGEN           segment AT 0B000h       ; dummy segment covers REGEN
REGEN           ends

;==============================================================

stack   segment stack           ; keep MS link happy by providing null stack
stack   ends

;==============================================================

CODE    SEGMENT PARA            ; start off in code.

;==============================================================

data    segment byte            ; provide a separate DATA segment
                                ; actually all come after the code

;==============================================================
;  V A R I A B L E S

PermitMode      DB      0       ; 0=Diag 1=Half 3=Full
ClearRam        DB      0       ; -1 means clear the REGEN, go to RAMFONT mode
HelpNeeded      DB      0       ; -1 means display the help screen

CardType        DB      0       ; 0=not herc 1=Herc 2=HercPlus
IsNotHerc       equ     0       ; equates for Cardtype
IsHerc          equ     1
IsHercPlus      equ     2

ttable          db      61h,50h,52h,0fh ; init parms for 6845 chip
                db      19h,06h,19h,19h
                db      02h,0dh,0bh,0ch

; M E S S A G E S

BannerMsg       label   byte
        DB      ' HGC! 1.4 ۲',13d,10d
        DB      13d,10d
        DB      'HGC! is a Hercules video mode utility replacement for HGC.',13,10
        DB      'For monochrome Hercules Graphics and Graphics Plus Video Cards only.',13,10
        DB      13,10
        DB      'Copyright (c) 1991-1999 Roedy Green Canadian Mind Products.',13,10
        DB      '#208 - 525 Ninth Street, New Westminster, BC Canada V3M 5T9',13,10
        DB      'tel:(604) 777-1804   mailto:roedy@mindprod.com   http://mindprod.com',13,10
        DB      'May be freely distributed and used for any purpose except military.',13,10,13,10
        db      '$'

UsageMsg        db 'Try:',13,10
                db 'HGC!         -- displays this help screen.',13,10
                db 'HGC! DIAG    -- to inhibit graphics.',13,10
                db 'HGC! HALF    -- to permit partial graphics.',13,10
                db 'HGC! FULL    -- to permit full graphics.',13,10
                db 'HGC! ROMFONT -- turns off fancy 4K or 48K RAMFONT modes',13,10
                db '                and also clears the screen.',13,10
                DB 13,10
                db 'Note, HGC! FULL does not actually put you in graphics mode;',13,10
                db 'it simply controls the lock that grants access to full graphics.',13,10
                db 'You may combine modes such as HGC! ROMFONT HALF.',13,10
                db '$'


NotHercMsg      db      0dh,0ah,'ERROR - Hercules Graphics Card not present',13,10
                db '$'


data            ends

;==============================================================

com     group   code,data       ; force data segment to go at the end!

        ASSUME  CS:com,DS:com,ES:com,SS:com
                                ; seg regs cover everything
        ORG     100H            ; in Code segment

;==============================================================

Start   proc    far
;       Mainline procedure
        Call    Parse           ; get mode parameter from command line
                                ; calls Analyse for each parameter
        Call    Detect          ; find out if a Herc Card is Present
        Test    CardType,IsHerc+IsHercPlus
        jz      Trouble

        Call    VideoOff        ; temporarily turn off video beam
        Call    SetPermission   ; set DIAG HALF FULL mode
        Call    Init6845        ; reprogram the 6845 CRT chip parms
        Test    ClearRam,-1
        jz      DontClear       ; bypass clearing screen if no ROMFONT parm

        Test    CardType,IsHercPlus
        jz      NoFancyStuff    ; bypass fancy reg setting for plain Herc

        Call    SetRomFont      ; turn off RAMFont, turn on std ROMFont
        Call    SetUnderline    ; set underline position
        Call    SetStrike       ; set overstrike position

NoFancyStuff:
        Call    ClearRegen      ; Clean the REGEN buffer
        Call    Home            ; Cursor to upper left

DontClear:
        Call    VideoOn         ; turn video beam back on
        test    HelpNeeded,-1
        jnz     Trouble
Done:
        mov     ax,4c00h
        int     21h             ; normal termination

Trouble:                        ; treated as HGC! DIAG ROMFONT + HELP!!
        lea     dx,BannerMsg    ; display the banner
        Call    Say
        lea     dx,UsageMsg     ; display usage message
        Call    Say
                                ; consider printing extra error mesage
        Test    CardType,IsHerc+IsHercPlus
        jnz     abort
        lea     dx,NotHercMsg   ; some other kind of card, MONO, CGA, VGA
        Call    Say

abort:
                                ; error exit
        mov  ax, 4c01h          ; ERRORLEVEL = 1
        int     21h             ; DIE

;       nested procs follow . . .

;===============================================================

Parse   Proc    near
;       Parse the command line for the optional  DIAG ROMFONT etc

        call    CommandLine     ; string addr ES:di, length cx.
        jcxz    NullParms

        mov     ParmIndex,1     ; start parsing with the 1st parm
                                ; note start with 1 not 0!
                                ; We don't want the the program name.
Parseloop:
        call    CommandLine     ; get first parm
                                ; string=ES:di, length=cx
        mov     bx,ParmIndex
        call    NthParm         ; work left to right
        jcxz    NoMoreParms     ; null param means no more
                                ; e.g. DS:si points to DIAG
        lodsb
                                ; al is D -- first letter of word
        call    Analyse         ; set variables for that letter
Next:
        inc     ParmIndex       ; bump loop counter
        jmp     Parseloop       ; loop till hit null param

NullParms:
        mov     al,020h         ; blank will trigger call for help.
        call    Analyse         ; force call for help
NoMoreParms:
        ret                     ; we are done

ParmIndex       dw      0
Parse   EndP

;==============================================================

CommandLine Proc        Near
;       Gets command line string into ES:DI
;       Command line does not include the program name.
;       ES already set since we are a COM file
                                ; counted string at HEX 80
                                ; contains command line.
                                ; It has no trailing null.
        MOV     DI,81h
        XOR     CH,CH
        MOV     CL,DS:80h       ; CX contains length of command
        RET
CommandLine EndP

;==============================================================

NthParm Proc    Near
;       Parses string for Nth Parameter delimited by blanks
;       e.g.  "  HALF DIAG" with BX=1 would give string "HALF"
;       ES:DI - string to be parsed for parameters
;       CX - length of string
;       BX - which parm wanted 1=parm1 2=parm2 etc.
;       NthParm only finds one parm per call.
;       On exit ES:SI points to string and CX is its length.
;       If there is no parm, the length will be 0.
;       It also handles multiple leading/trailing blanks on parms.
        MOV     AL,20h          ; AL = blank  -- the search char
ParmLoop:
;       Remove leading blanks on parm
        JCXZ    NullParm        ; jump if null string
        REPE    SCASB           ; scan ES:DI forwards till hit non blank
                                ; DI points just after it
                                ; CX is one too small, or 0 if none found
        JE      NullParm        ; jump if entire string was blank
        INC     CX              ; CX is length of remainder of string
        DEC     DI              ; DI points to non-blank
        MOV     SI,DI           ; remember start of string
;       Search for terminating blank on parm
        JCXZ    NullParm        ; jump if null string
        REPNE   SCASB           ; scan ES:DI forwards till hit blank
                                ; DI points just after it
                                ; CX is one too small, or 0 if none found
        JNE     NoBlank         ; jump if entire string was non blank
        INC     CX              ; CX is length of remainder of string
        DEC     DI              ; backup DI to point to blank at string end
NoBlank:
                                ; DI=addr tail end of command string,
                                ; CX=len tail end of command string
                                ; SI=addr parm just parsed
;       Major loop for each parm
        DEC     BX
        JNZ     ParmLoop        ; loop once for each parm

        MOV     CX,DI
        SUB     CX,SI           ; CX is length of parameter.
        RET
NullParm:                       ; was no nth parameter
        MOV     CX,0
        RET
NthParm EndP


;==============================================================

Analyse Proc    Near
;       Analyse the one parameter just presented
;       Leave results in internal format
;       e.g. PermitMode  ClearRam and HelpNeeded
;       First letter of command word is in AL

        call    ToUc            ; convert it to upper case

        cmp     al,"D"
        jne     NotDiag
;                                       ; HGC! DIAG
        mov     PermitMode,0
        ret
NotDiag:

        cmp     al,"H"
        jne     NotHalf
;                                       ; HGC! HALF
        mov     PermitMode,1
        ret
NotHalf:

        cmp     al,"F"
        jne     NotFull
;                                       ; HGC! FULL
        mov     PermitMode,3
        ret
NotFull:

        cmp     al,"R"
        jne     NotRAM
;                                       ; HGC! ROMFONT
        mov     ClearRam,-1
        ret
NotRam:

;       Something else                  ; treat as HGC! ROMFONT DIAG + HELP!
        mov     ClearRam,-1
        mov     HelpNeeded,-1
        ret
;                                       ; leave permitmode, to allow
;                                       ; HGC! FULL ROMFONT
Analyse EndP

;==============================================================

ToUC    Proc    near
;       converts char in al to upper case
        cmp     al,'a'
        jb      FineAsIs
        cmp     al,'z'
        ja      FineAsIs
        sub     al,20H          ; convert a to A
FineAsIs:
        ret
ToUc    EndP

;==============================================================

Detect  proc    near

; Performs two tests - first, makes sure that a Hercules
; Graphics Card is present and active, then checks for a GB112.
; Set Flag CardType 0=not Herc, 1=plan Graphics 2=HercPlus GB112

; we do NOT test the current video state.  We detect the card even
; in a dual monitor system with the CGA card active.

        mov     dx,StatusPort           ; record state
        in      al,dx
        and     al,80h                  ; save bit 7 for test
        mov     ah,al

        mov     cx,8000h

examine:
        in      al,dx                   ; take another reading
        and     al,80h                  ; again save bit seven
        cmp     al,ah
        jne     hgc                     ; if bit 7 changes, then it
        loop    examine                 ; is a Hercules Graphics Card

        mov     CardType,0
        ret                             ; after this long it must be
                                        ; something else - quit.

hgc:    in      al,dx                   ; test for GB112
        and     al,id_mask              ; clear all but bits 4 and 5

        cmp     al,id_code              ; test ID bits
        jne     t_f_1                   ; exit if failed - not a GB112
        mov     CardType,2              ; IS a HERC PLUS GB112
        ret                             ; normal return

t_f_1:
        mov     CardType,1              ; is a plain Herc
        ret

Detect  EndP

;==============================================================

VideoOff        Proc    Near

;       temporarily blank the screen while we change parameters
        mov     dx,CntrlPort
        mov     ax,TextMode+scrn_off
        out     dx,al
        ret

VideoOff        EndP

;==============================================================

VideoOn Proc    Near

;       restore video beam
        mov     dx,CntrlPort
        mov     ax,TextMode+scrn_on
        out     dx,al
        ret

VideoOn         EndP

;==============================================================

SetPermission   Proc    Near

;       Set the permission mode DIAG HALF or FULL

        mov     al,PermitMode
        mov     dx,AllowPort
        out     dx,al
        ret
SetPermission   EndP

;==============================================================

Init6845        Proc    Near

;     initialize the 6845, sending it 12 parameters

        mov     dx,IndexPort
        mov     cx,12           ;12 parameters to
                                ;be output
        xor     ah,ah           ;starting from
                                ;reg. 0
        lea     si,ttable       ;get address of list of parameters

parms:  mov     al,ah
        out     dx,al           ;output register
                                ;number
        inc     dx
        lodsb
        out     dx,al           ;output data

        inc     ah              ;next value
        dec     dx
        loop    parms
        ret

Init6845        EndP

;==============================================================

SetROMFont      Proc    Near
;       Turn off RAMFont mode, make card emulate standard MONO card
        mov     dx,IndexPort
        mov     al,014h         ; select the mode register
        out     dx,al
        mov     dx,DataPort
        mov     al,0            ; code for 4K 9bit ROMFont
        out     dx,al
        ret
SetROMFont      EndP

;==============================================================

SetUnderline    Proc    Near
;   Select underline position
        mov     dx,Indexport
        mov     al,015h         ; score register
        out     dx,al
        mov     dx,DataPort
        mov     al,13d          ; put on bottom line # 13
        out     dx,al
        ret
SetUnderline    EndP

;==============================================================

SetStrike       Proc    Near
;   Select underline position
        mov     dx,Indexport
        mov     al,016h         ; overstrike register
        out     dx,al
        mov     dx,DataPort
        mov     al,7d           ; half-way up line 7.
        out     dx,al
        ret
SetStrike       EndP

;==============================================================

ClearREGEN      Proc    Near

; Clear the REGEN buffer.  Snow is no problem.
; If we left display weird RAMFONT attributes could look baffling.
        push    ES
        mov     cx,80*25        ; number of chars on screen in words
        mov     dx,SEG REGEN
        mov     ES,dx
        xor     di,di           ; ES:DI points to start of REGEN
        mov     ax,0720h        ; blank+vanilla attribute byte
        rep stosw
        pop     ES
        ret

ClearREGEN      EndP

;==============================================================

Home    Proc    Near

;       Moves cursor to upper left.
;       Check current video mode.  If not using herc, we had better
;       leave the cursor alone.
        mov     ah,15           ; check current video state
        int     10h

        cmp     al,7            ; is Herc mode active VMODE 7?
        je      OkToHome
        ret                     ; no - do nothing

OkToHome:
        xor     dx,dx           ; DH:DL = row:col
        mov     bx,dx           ; bh=page 0
        mov     ah,2            ; code for move cursor
        int     010h            ; video BIOS interrupt
        ret

Home    EndP

;==============================================================

Say     Proc
;       on entry DX points to a string to display
        MOV     AH,9
        Int     21h
        ret
Say     EndP

;==============================================================

Start   endp

;==========================
CODE    ends                    ; end of code segment
        end     Start
